require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/digital_workspace/calendar/calendar_get_busy_time_list'
require 'uu/digital_workspace/calendar/calendar_busy_time_attributes'

module UU
  module DigitalWorkspace
    module Calendar
      extend self

      # Service path
      PATH = 'uu/os/Calendar'

      # Returns list of digital workspace calendar blocks for the specified user according to the specified parameters time_min and time_max. The maximum time difference between time_min and time_max is six weeks.
      # 
      # @param [String, UU::OS::UESURI] role_uri <b>Not nil.</b> URI of the role, access role or personal role. User's list of time blocks in the calendar is derived from the primary cast in case that role is entered and access_role_uri is not. User is known in case that access role or personal role is entered. Parameter must be filled, otherwise the command fails.
      # @param [UU::DigitalWorkspace::Calendar::CalendarGetBusyTimeList] criteria Specific options.
      # @option criteria [String, UU::OS::UESURI] :access_role_uri URI of the user's access role. Entered access role must be cast to the role specified in command main entity. Parameter can be used if the main entity is the role (not Access Role, not Personal Role).
      # @option criteria [String] :time_min <b>Not nil.</b> The start of the time interval, where blocks of time with date_start equal or later than entered value in this parameter will be returned in the result. The entered value must be in ISO 8601 format.
      # @option criteria [String] :time_max <b>Not nil.</b> The end of the time interval, where blocks of time with date_end equal or former than entered value in this parameter will be returned in the result. The entered value must be in ISO 8601 format.
      #
      # @return [UU::OS::REST::ResultList<Calendar::CalendarGetBusyTimeList, Calendar::CalendarBusyTimeAttributes>] List of digital workspace calendar blocks for the specified users. Items in the list are returned according to the entered time_min and time_max. Contains instances of {UU::DigitalWorkspace::Calendar::CalendarBusyTimeAttributes}.
      #
      # @example
      #   UU::DigitalWorkspace::Calendar.get_busy_time_list('ues:TERRITORY:ROLE',
      #      :access_role_uri => "ues:TERRITORY:ACCESS_ROLE",
      #      :time_min => "2014-07-01T09:00:00.000+01:00",
      #      :time_max => "2014-07-31T18:00:00.000+01:00")
      def get_busy_time_list(role_uri, criteria)
        svc = UU::OS::REST::RemoteClient.new(Calendar)
        dto = UU::DigitalWorkspace::Calendar::CalendarGetBusyTimeList.new(criteria)

        svc.add_parameter('timeMin', dto.time_min)
        svc.add_parameter('timeMax', dto.time_max)
        svc.add_parameter('accessRoleUri', dto.access_role_uri)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getBusyTimeList', UU::OS::UESURI.new(role_uri))
          return UU::OS::REST::ResultList.new(Calendar::CalendarGetBusyTimeList, Calendar::CalendarBusyTimeAttributes, res)
        end
      end

    end
  end
end